@@ -268,8 +268,7 @@ sub tag_open
     $$self !~ /\/>$/ and
     $$self !~ /^<\?/ and    
     $$self =~ /^</   and do {
-	my %node      = ($$self =~ /((?:\w|:|-)+)\s*=\s*\"(.*?)\"/gs,
-	                 $$self =~ /((?:\w|:|-)+)\s*=\s*\'(.*?)\'/gs);
+	my %node      = _extract_attributes ($$self);
 	($node{_tag}) = $$self =~ /.*?([A-Za-z0-9][A-Za-z0-9_:-]*)/;
 	$node{_open}  = 1;
 	$node{_close} = 0;
@@ -337,10 +336,10 @@ sub tag_self_close
     $$self !~ /^<\!/ and
     $$self !~ /^<\// and
     $$self =~ /\/>$/ and
+    # ((?:\w|:|-)+)\s*=\s*\"(.*?)\"/gs;
     $$self =~ /^</   and do {
-	my %node      = ($$self =~ /((?:\w|:|-)+)\s*=\s*\"(.*?)\"/gs,
-	                 $$self =~ /((?:\w|:|-)+)\s*=\s*\'(.*?)\'/gs);
-	($node{_tag}) = $$self =~ /.*?([A-Za-z0-9][A-Za-z0-9_:-]*)/;
+	my %node      = _extract_attributes ($$self);
+        ($node{_tag}) = $$self =~ /.*?([A-Za-z0-9][A-Za-z0-9_:-]*)/;
 	$node{_open}  = 1;
 	$node{_close} = 1;
 	$node = \%node;
@@ -367,6 +366,35 @@ sub text
     return ($$self !~ /^</) ? $$self : undef;
 }
 
+our $S = "[ \\n\\t\\r]+";
+our $NameStrt = "[A-Za-z_:]|[^\\x00-\\x7F]";
+our $NameChar = "[A-Za-z0-9_:.-]|[^\\x00-\\x7F]";
+our $Name = "(?:$NameStrt)(?:$NameChar)*";
+our $EndTagCE = "$Name(?:$S)?>?";
+our $AttValSE = "\"[^<\"]*\"|'[^<']*'";
+our $ElemTagCE = "$Name((?:$S$Name(?:$S)?=(?:$S)?(?:$AttValSE))*)(?:$S)?/?>?";
+our $ElemTagCE_Mod = "$S($Name)(?:$S)?=(?:$S)?($AttValSE)";
+
+our $RE_1 = qr /$ElemTagCE/;
+our $RE_2 = qr /$ElemTagCE_Mod/;
+
+
+sub _extract_attributes
+{
+    my $tag = shift;
+    my ($tags) = $tag =~ /$RE_1/g;
+    my %attr = $tag =~ /$RE_2/g;
+    foreach my $key (keys %attr)
+    {
+        my $val = $attr{$key};
+        $val    =~ s/^(\"|\')//;
+        $val    =~ s/(\"|\')$//;
+        $attr{$key} = $val;
+    }
+    
+    %attr;
+}
+
 
 1;
 
